module win32.uxtheme;

private import win32.commctrl;
private import win32.w32api, win32.windef, win32.winuser;
private import win32.wingdi;
private import win32.winbase; // for SYSTEMTIME
private import win32.objfwd;  // for LPSTREAM

alias HANDLE HTHEME;

static if (_WIN32_WINNT >= 0x0600) {

  enum : BP_BUFFERFORMAT {
    BPBF_COMPATIBLEBITMAP,
    BPBF_DIB,
    BPBF_TOPDOWNDIB,
    BPBF_TOPDOWNMONODIB
  }

  struct BP_PAINTPARAMS {
    DWORD cbSize = BP_PAINTPARAMS.sizeof;
    DWORD dwFlags;
    const RECT *prcExclude;
    const BLENDFUNCTION *pBlendFunction;
  }
  alias BP_PAINTPARAMS* PBP_PAINTPARAMS;

  extern(Windows) {
    HPAINTBUFFER BeginBufferedPaint(HDC hdcTarget,const RECT *prcTarget,BP_BUFFERFORMAT dwFormat,
                                    BP_PAINTPARAMS *pPaintParams,HDC *phdc);
    HRESULT EndBufferedPaint(HPAINTBUFFER hBufferedPaint,BOOL fUpdateTarget);
    HRESULT GetBufferedPaintTargetRect(HPAINTBUFFER hBufferedPaint,RECT *prc);
    HDC GetBufferedPaintTargetDC(HPAINTBUFFER hBufferedPaint);
    HDC GetBufferedPaintDC(HPAINTBUFFER hBufferedPaint);
    HRESULT GetBufferedPaintBits(HPAINTBUFFER hBufferedPaint,RGBQUAD **ppbBuffer,int *pcxRow);
    HRESULT BufferedPaintClear(HPAINTBUFFER hBufferedPaint,const RECT *prc);
    HRESULT BufferedPaintSetAlpha(HPAINTBUFFER hBufferedPaint,const RECT *prc,BYTE alpha);
    HRESULT BufferedPaintInit(VOID);
    HRESULT BufferedPaintUnInit(VOID);
  }

}

extern(Windows) {
  HTHEME  OpenThemeData(HWND hwnd,LPCWSTR pszClassList);
  HRESULT CloseThemeData(HTHEME hTheme);
  HRESULT DrawThemeBackground(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,const RECT *pRect,const RECT *pClipRect);
}

const WORD DTT_GRAYED = 0x1 ;

enum THEMESIZE {
  TS_MIN,
  TS_TRUE,
  TS_DRAW
}

extern(Windows){
  HRESULT GetThemePartSize(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,RECT *prc,THEMESIZE eSize,SIZE *psz);
  HRESULT GetThemeTextExtent(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,LPCWSTR pszText,int iCharCount,DWORD dwTextFlags,const RECT *pBoundingRect,RECT *pExtentRect);
  HRESULT GetThemeTextMetrics(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,TEXTMETRIC *ptm);
  HRESULT GetThemeBackgroundRegion(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,const RECT *pRect,HRGN *pRegion);
}

const WORD HTTB_BACKGROUNDSEG = 0x0000 ;
const WORD HTTB_FIXEDBORDER   = 0x0002 ;
const WORD HTTB_CAPTION       = 0x0004 ;
const WORD HTTB_RESIZINGBORDER_LEFT  = 0x0010 ;
const WORD HTTB_RESIZINGBORDER_TOP   = 0x0020 ;
const WORD HTTB_RESIZINGBORDER_RIGHT = 0x0040 ;
const WORD HTTB_RESIZINGBORDER_BOTTOM= 0x0080 ;

const WORD HTTB_RESIZINGBORDER = (HTTB_RESIZINGBORDER_LEFT|HTTB_RESIZINGBORDER_TOP| HTTB_RESIZINGBORDER_RIGHT|HTTB_RESIZINGBORDER_BOTTOM) ;

const WORD HTTB_SIZINGTEMPLATE      = 0x0100 ;
const WORD HTTB_SYSTEMSIZINGMARGINS = 0x0200 ;

extern(Windows)
{
  HRESULT HitTestThemeBackground(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,DWORD dwOptions,const RECT *pRect,HRGN hrgn,POINT ptTest,WORD *pwHitTestCode);
  HRESULT DrawThemeEdge(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,const RECT *pDestRect,UINT uEdge,UINT uFlags,RECT *pContentRect);
  HRESULT DrawThemeIcon(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,const RECT *pRect,HIMAGELIST himl,int iImageIndex);
  WINBOOL IsThemePartDefined(HTHEME hTheme,int iPartId,int iStateId);
  WINBOOL IsThemeBackgroundPartiallyTransparent(HTHEME hTheme,int iPartId,int iStateId);
  HRESULT GetThemeColor(HTHEME hTheme,int iPartId,int iStateId,int iPropId,COLORREF *pColor);
  HRESULT GetThemeMetric(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,int iPropId,int *piVal);
  HRESULT GetThemeString(HTHEME hTheme,int iPartId,int iStateId,int iPropId,LPWSTR pszBuff,int cchMaxBuffChars);
  HRESULT GetThemeBool(HTHEME hTheme,int iPartId,int iStateId,int iPropId,WINBOOL *pfVal);
  HRESULT GetThemeInt(HTHEME hTheme,int iPartId,int iStateId,int iPropId,int *piVal);
  HRESULT GetThemeEnumValue(HTHEME hTheme,int iPartId,int iStateId,int iPropId,int *piVal);
  HRESULT GetThemePosition(HTHEME hTheme,int iPartId,int iStateId,int iPropId,POINT *pPoint);
  HRESULT GetThemeFont(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,int iPropId,LOGFONT *pFont);
  HRESULT GetThemeRect(HTHEME hTheme,int iPartId,int iStateId,int iPropId,RECT *pRect);
}

struct MARGINS {
  int cxLeftWidth;
  int cxRightWidth;
  int cyTopHeight;
  int cyBottomHeight;
}
alias MARGINS* PMARGINS;

extern(Windows)
{
  HRESULT GetThemeMargins(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,int iPropId,RECT *prc,MARGINS *pMargins);
}

const WORD MAX_INTLIST_COUNT = 10 ;

struct INTLIST {
  int iValueCount;
  int iValues[MAX_INTLIST_COUNT];
}
alias INTLIST* PINTLIST;

extern(Windows)
{
  HRESULT GetThemeIntList(HTHEME hTheme,int iPartId,int iStateId,int iPropId,INTLIST *pIntList);
}

enum PROPERTYORIGIN {
  PO_STATE,
  PO_PART,
  PO_CLASS,
  PO_GLOBAL,
  PO_NOTFOUND
} ;

extern(Windows)
{
  HRESULT GetThemePropertyOrigin(HTHEME hTheme,int iPartId,int iStateId,int iPropId,PROPERTYORIGIN *pOrigin);
  HRESULT SetWindowTheme(HWND hwnd,LPCWSTR pszSubAppName,LPCWSTR pszSubIdList);
  HRESULT GetThemeFilename(HTHEME hTheme,int iPartId,int iStateId,int iPropId,LPWSTR pszThemeFileName,int cchMaxBuffChars);
  COLORREF GetThemeSysColor(HTHEME hTheme,int iColorId);
  HBRUSH   GetThemeSysColorBrush(HTHEME hTheme,int iColorId);
  WINBOOL  GetThemeSysBool(HTHEME hTheme,int iBoolId);
  int      GetThemeSysSize(HTHEME hTheme,int iSizeId);
  HRESULT  GetThemeSysFont(HTHEME hTheme,int iFontId,LOGFONT *plf);
  HRESULT  GetThemeSysString(HTHEME hTheme,int iStringId,LPWSTR pszStringBuff,int cchMaxStringChars);
  HRESULT  GetThemeSysInt(HTHEME hTheme,int iIntId,int *piValue);
  WINBOOL  IsThemeActive();
  WINBOOL  IsAppThemed();
  HTHEME   GetWindowTheme(HWND hwnd);
}

const WORD ETDT_DISABLE = 0x00000001 ;
const WORD ETDT_ENABLE  = 0x00000002 ;
const WORD ETDT_USETABTEXTURE = 0x00000004 ;
const WORD ETDT_ENABLETAB = (ETDT_ENABLE | ETDT_USETABTEXTURE) ;

extern(Windows)
{
  HRESULT EnableThemeDialogTexture(HWND hwnd,DWORD dwFlags);
  WINBOOL IsThemeDialogTextureEnabled(HWND hwnd);
}

const WORD STAP_ALLOW_NONCLIENT  = (1 << 0) ;
const WORD STAP_ALLOW_CONTROLS   = (1 << 1) ;
const WORD STAP_ALLOW_WEBCONTENT = (1 << 2) ;

extern(Windows)
{
  DWORD   GetThemeAppProperties();
  void    SetThemeAppProperties(DWORD dwFlags);
  HRESULT GetCurrentThemeName(LPWSTR pszThemeFileName,int cchMaxNameChars,LPWSTR pszColorBuff,int cchMaxColorChars,LPWSTR pszSizeBuff,int cchMaxSizeChars);
}

const TCHAR[]
  SZ_THDOCPROP_DISPLAYNAME   = "DisplayName",
  SZ_THDOCPROP_CANONICALNAME = "ThemeName" ,
  SZ_THDOCPROP_TOOLTIP       = "ToolTip",
  SZ_THDOCPROP_AUTHOR        = "author" ;

extern(Windows)
{
  HRESULT GetThemeDocumentationProperty(LPCWSTR pszThemeName,LPCWSTR pszPropertyName,LPWSTR pszValueBuff,int cchMaxValChars);
  HRESULT DrawThemeParentBackground(HWND hwnd,HDC hdc,RECT *prc);
  HRESULT EnableTheming(WINBOOL fEnable);
}

const WORD DTBG_CLIPRECT    = 0x00000001 ;
const WORD DTBG_DRAWSOLID   = 0x00000002 ;
const WORD DTBG_OMITBORDER  = 0x00000004 ;
const WORD DTBG_OMITCONTENT = 0x00000008 ;
const WORD DTBG_COMPUTINGREGION = 0x00000010 ;
const WORD DTBG_MIRRORDC        = 0x00000020 ;

struct DTBGOPTS {
  DWORD dwSize;
  DWORD dwFlags;
  RECT rcClip;
}
alias DTBGOPTS* PDTBGOPTS;

extern(Windows)
{
  HRESULT DrawThemeBackgroundEx(HTHEME hTheme,HDC hdc,int iPartId,int iStateId,const RECT *pRect,const DTBGOPTS *pOptions);
}
